@@ -5,6 +5,11 @@
 readstat_error_t readstat_begin_writing_file(readstat_writer_t *writer, void *user_ctx, long row_count);
 
 readstat_error_t readstat_write_bytes(readstat_writer_t *writer, const void *bytes, size_t len);
+readstat_error_t readstat_write_bytes_as_lines(readstat_writer_t *writer,
+        const void *bytes, size_t len, size_t line_len, const char *line_sep);
+readstat_error_t readstat_write_line_padding(readstat_writer_t *writer, char pad,
+        size_t line_len, const char *line_sep);
+
 readstat_error_t readstat_write_zeros(readstat_writer_t *writer, size_t len);
 readstat_error_t readstat_write_spaces(readstat_writer_t *writer, size_t len);
 readstat_error_t readstat_write_string(readstat_writer_t *writer, const char *bytes);
