@@ -310,8 +310,10 @@ static readstat_error_t sas7bdat_parse_column_format_subheader(const char *subhe
     return retval;
 }
 
-static readstat_error_t sas7bdat_handle_data_value(const char *col_data, col_info_t *col_info, sas7bdat_ctx_t *ctx) {
+static readstat_error_t sas7bdat_handle_data_value(readstat_variable_t *variable, 
+        col_info_t *col_info, const char *col_data, sas7bdat_ctx_t *ctx) {
     readstat_error_t retval = READSTAT_OK;
+    char error_buf[ERROR_BUF_SIZE];
     int cb_retval = 0;
     readstat_value_t value;
     memset(&value, 0, sizeof(readstat_value_t));
@@ -321,8 +323,15 @@ static readstat_error_t sas7bdat_handle_data_value(const char *col_data, col_inf
     if (col_info->type == READSTAT_TYPE_STRING) {
         retval = readstat_convert(ctx->scratch_buffer, ctx->scratch_buffer_len,
                 col_data, col_info->width, ctx->converter);
-        if (retval != READSTAT_OK)
+        if (retval != READSTAT_OK) {
+            if (ctx->error_handler) {
+                snprintf(error_buf, sizeof(error_buf),
+                        "ReadStat: Error converting string to specified encoding: %.*s\n",
+                        col_info->width, col_data);
+                ctx->error_handler(error_buf, ctx->user_ctx);
+            }
             goto cleanup;
+        }
 
         value.v.string_value = ctx->scratch_buffer;
     } else if (col_info->type == READSTAT_TYPE_DOUBLE) {
@@ -355,10 +364,9 @@ static readstat_error_t sas7bdat_handle_data_value(const char *col_data, col_inf
             value.v.double_value = dval;
         }
     }
-    cb_retval = ctx->value_handler(ctx->parsed_row_count, ctx->variables[col_info->index], 
-            value, ctx->user_ctx);
+    cb_retval = ctx->value_handler(ctx->parsed_row_count, variable, value, ctx->user_ctx);
 
-    if (cb_retval)
+    if (cb_retval != READSTAT_HANDLER_OK)
         retval = READSTAT_ERROR_USER_ABORT;
 
 cleanup:
@@ -376,7 +384,11 @@ static readstat_error_t sas7bdat_parse_single_row(const char *data, sas7bdat_ctx
         ctx->scratch_buffer = realloc(ctx->scratch_buffer, ctx->scratch_buffer_len);
         for (j=0; j<ctx->column_count; j++) {
             col_info_t *col_info = &ctx->col_info[j];
-            retval = sas7bdat_handle_data_value(&data[col_info->offset], col_info, ctx);
+            readstat_variable_t *variable = ctx->variables[j];
+            if (variable->skip)
+                continue;
+
+            retval = sas7bdat_handle_data_value(variable, col_info, &data[col_info->offset], ctx);
             if (retval != READSTAT_OK) {
                 goto cleanup;
             }
@@ -469,11 +481,13 @@ cleanup:
     return retval;
 }
 
-static readstat_variable_t *sas7bdat_init_variable(sas7bdat_ctx_t *ctx, int i, readstat_error_t *out_retval) {
+static readstat_variable_t *sas7bdat_init_variable(sas7bdat_ctx_t *ctx, int i, 
+        int index_after_skipping, readstat_error_t *out_retval) {
     readstat_error_t retval = READSTAT_OK;
     readstat_variable_t *variable = calloc(1, sizeof(readstat_variable_t));
 
     variable->index = i;
+    variable->index_after_skipping = index_after_skipping;
     variable->type = ctx->col_info[i].type;
     variable->storage_width = ctx->col_info[i].width;
 
@@ -496,6 +510,16 @@ cleanup:
         if (out_retval)
             *out_retval = retval;
 
+        if (retval == READSTAT_ERROR_CONVERT_BAD_STRING) {
+            if (ctx->error_handler) {
+                char error_buf[ERROR_BUF_SIZE];
+                snprintf(error_buf, sizeof(error_buf),
+                        "ReadStat: Error converting variable #%d info to specified encoding: %s %s (%s)\n",
+                        i, variable->name, variable->format, variable->label);
+                ctx->error_handler(error_buf, ctx->user_ctx);
+            }
+        }
+
         return NULL;
     }
 
@@ -505,29 +529,38 @@ cleanup:
 static readstat_error_t sas7bdat_submit_columns(sas7bdat_ctx_t *ctx) {
     readstat_error_t retval = READSTAT_OK;
     if (ctx->info_handler) {
-        if (ctx->info_handler(ctx->row_limit, ctx->column_count, ctx->user_ctx)) {
+        if (ctx->info_handler(ctx->row_limit, ctx->column_count, ctx->user_ctx) != READSTAT_HANDLER_OK) {
             retval = READSTAT_ERROR_USER_ABORT;
             goto cleanup;
         }
     }
     if (ctx->metadata_handler) {
-        if (ctx->metadata_handler(ctx->file_label, ctx->timestamp, ctx->version, ctx->user_ctx)) {
+        if (ctx->metadata_handler(ctx->file_label, ctx->timestamp, ctx->version, ctx->user_ctx) != READSTAT_HANDLER_OK) {
             retval = READSTAT_ERROR_USER_ABORT;
             goto cleanup;
         }
     }
     ctx->variables = calloc(ctx->column_count, sizeof(readstat_variable_t *));
     int i;
+    int index_after_skipping = 0;
     for (i=0; i<ctx->column_count; i++) {
-        ctx->variables[i] = sas7bdat_init_variable(ctx, i, &retval);
+        ctx->variables[i] = sas7bdat_init_variable(ctx, i, index_after_skipping, &retval);
         if (ctx->variables[i] == NULL)
             break;
 
+        int cb_retval = READSTAT_HANDLER_OK;
         if (ctx->variable_handler) {
-            if (ctx->variable_handler(i, ctx->variables[i], ctx->variables[i]->format, ctx->user_ctx)) {
-                retval = READSTAT_ERROR_USER_ABORT;
-                goto cleanup;
-            }
+            cb_retval = ctx->variable_handler(i, ctx->variables[i], ctx->variables[i]->format, ctx->user_ctx);
+        }
+        if (cb_retval == READSTAT_HANDLER_ABORT) {
+            retval = READSTAT_ERROR_USER_ABORT;
+            goto cleanup;
+        }
+
+        if (cb_retval == READSTAT_HANDLER_SKIP_VARIABLE) {
+            ctx->variables[i]->skip = 1;
+        } else {
+            index_after_skipping++;
         }
     }
 cleanup:
