@@ -144,7 +144,18 @@ static readstat_error_t sas7bdat_emit_header(readstat_writer_t *writer, sas_head
     };
 
     memcpy(&header_start.magic, sas7bdat_magic_number, sizeof(header_start.magic));
-    strncpy(header_start.file_label, writer->file_label, sizeof(header_start.file_label));
+
+    memset(header_start.file_label, ' ', sizeof(header_start.file_label));
+
+    size_t file_label_len = strlen(writer->file_label);
+    if (file_label_len > sizeof(header_start.file_label))
+        file_label_len = sizeof(header_start.file_label);
+
+    if (file_label_len) {
+        memcpy(header_start.file_label, writer->file_label, file_label_len);
+    } else {
+        memcpy(header_start.file_label, "DATASET", sizeof("DATASET")-1);
+    }
 
     return sas_write_header(writer, hinfo, header_start);
 }
@@ -202,34 +213,6 @@ static sas7bdat_subheader_t *sas7bdat_col_size_subheader_init(readstat_writer_t
     return subheader;
 }
 
-static readstat_error_t sas7bdat_validate_name(const char *name) {
-    int j;
-    for (j=0; name[j]; j++) {
-        if (name[j] != '_' &&
-                !(name[j] >= 'a' && name[j] <= 'z') &&
-                !(name[j] >= 'A' && name[j] <= 'Z') &&
-                !(name[j] >= '0' && name[j] <= '9')) {
-            return READSTAT_ERROR_NAME_CONTAINS_ILLEGAL_CHARACTER;
-        }
-    }
-    char first_char = name[0];
-    if (first_char != '_' &&
-            !(first_char >= 'a' && first_char <= 'z') &&
-            !(first_char >= 'A' && first_char <= 'Z')) {
-        return READSTAT_ERROR_NAME_BEGINS_WITH_ILLEGAL_CHARACTER;
-    }
-    if (strcmp(name, "_N_") == 0 || strcmp(name, "_ERROR_") == 0 ||
-            strcmp(name, "_NUMERIC_") == 0 || strcmp(name, "_CHARACTER_") == 0 ||
-            strcmp(name, "_ALL_") == 0) {
-        return READSTAT_ERROR_NAME_IS_RESERVED_WORD;
-    }
-
-    if (strlen(name) > 32)
-        return READSTAT_ERROR_NAME_IS_TOO_LONG;
-
-    return READSTAT_OK;
-}
-
 static sas7bdat_subheader_t *sas7bdat_col_name_subheader_init(readstat_writer_t *writer,
         sas_header_info_t *hinfo, sas7bdat_column_text_array_t *column_text_array) {
     size_t len = (hinfo->u64 ? 28+8*writer->variables_count :
@@ -430,8 +413,14 @@ static readstat_error_t sas7bdat_emit_meta_pages(readstat_writer_t *writer) {
 
         memcpy(&page[hinfo->page_header_size-8], &page_type, sizeof(int16_t));
 
+        if (sarray->subheaders[shp_written]->len + shp_ptr_size >
+                shp_data_offset - shp_ptr_offset) {
+            retval = READSTAT_ERROR_ROW_IS_TOO_WIDE_FOR_PAGE;
+            goto cleanup;
+        }
+
         while (sarray->count > shp_written && 
-                sarray->subheaders[shp_written]->len + shp_ptr_size <
+                sarray->subheaders[shp_written]->len + shp_ptr_size <=
                 shp_data_offset - shp_ptr_offset) {
             sas7bdat_subheader_t *subheader = sarray->subheaders[shp_written];
             uint32_t signature32 = subheader->signature;
@@ -484,6 +473,7 @@ static readstat_error_t sas7bdat_emit_meta_pages(readstat_writer_t *writer) {
             shp_written++;
             shp_count++;
         }
+
         if (hinfo->u64) {
             memcpy(&page[34], &shp_count, sizeof(int16_t));
             memcpy(&page[36], &shp_count, sizeof(int16_t));
@@ -503,17 +493,6 @@ cleanup:
     return retval;
 }
 
-static readstat_error_t sas7bdat_validate_column_names(readstat_writer_t *writer) {
-    int i;
-    for (i=0; i<writer->variables_count; i++) {
-        readstat_variable_t *variable = readstat_get_variable(writer, i);
-        readstat_error_t error = sas7bdat_validate_name(readstat_variable_get_name(variable));
-        if (error != READSTAT_OK)
-            return error;
-    }
-    return READSTAT_OK;
-}
-
 static sas7bdat_write_ctx_t *sas7bdat_write_ctx_init(readstat_writer_t *writer) {
     sas7bdat_write_ctx_t *ctx = calloc(1, sizeof(sas7bdat_write_ctx_t));
 
@@ -533,6 +512,12 @@ static readstat_error_t sas7bdat_emit_header_and_meta_pages(readstat_writer_t *w
     sas7bdat_write_ctx_t *ctx = (sas7bdat_write_ctx_t *)writer->module_ctx;
     readstat_error_t retval = READSTAT_OK;
 
+    if (writer->compression == READSTAT_COMPRESS_NONE &&
+            sas7bdat_rows_per_page(writer, ctx->hinfo) == 0) {
+        retval = READSTAT_ERROR_ROW_IS_TOO_WIDE_FOR_PAGE;
+        goto cleanup;
+    }
+
     ctx->hinfo->page_count = sas7bdat_count_meta_pages(writer) + sas7bdat_count_data_pages(writer, ctx->hinfo);
 
     retval = sas7bdat_emit_header(writer, ctx->hinfo);
@@ -551,7 +536,7 @@ static readstat_error_t sas7bdat_begin_data(void *writer_ctx) {
     readstat_writer_t *writer = (readstat_writer_t *)writer_ctx;
     readstat_error_t retval = READSTAT_OK;
 
-    retval = sas7bdat_validate_column_names(writer);
+    retval = sas_validate_column_names(writer);
     if (retval != READSTAT_OK)
         goto cleanup;
 
@@ -622,10 +607,10 @@ static readstat_error_t sas7bdat_write_missing_tagged_raw(void *row, const reads
 }
 
 static readstat_error_t sas7bdat_write_missing_tagged(void *row, const readstat_variable_t *var, char tag) {
-    if (tag < 'a' || tag > 'z')
-        return READSTAT_ERROR_TAGGED_VALUE_IS_OUT_OF_RANGE;
+    if (tag == '_' || (tag >= 'A' && tag <= 'Z'))
+        return sas7bdat_write_missing_tagged_raw(row, var, tag);
 
-    return sas7bdat_write_missing_tagged_raw(row, var, tag);
+    return READSTAT_ERROR_TAGGED_VALUE_IS_OUT_OF_RANGE;
 }
 
 static readstat_error_t sas7bdat_write_missing_numeric(void *row, const readstat_variable_t *var) {
