@@ -396,8 +396,12 @@ static readstat_error_t sav_submit_value_labels(value_label_t *value_labels, int
 
             value.v.string_value = unpadded_val;
         }
-        ctx->value_label_handler(label_name_buf, value, vlabel->label, ctx->user_ctx);
+        if (ctx->value_label_handler(label_name_buf, value, vlabel->label, ctx->user_ctx) != READSTAT_HANDLER_OK) {
+            retval = READSTAT_ERROR_USER_ABORT;
+            goto cleanup;
+        }
     }
+cleanup:
     return retval;
 }
 
@@ -541,7 +545,7 @@ static readstat_error_t sav_read_document_record(sav_ctx_t *ctx) {
         if (retval != READSTAT_OK)
             goto cleanup;
 
-        if (ctx->note_handler(i, utf8_buffer, ctx->user_ctx)) {
+        if (ctx->note_handler(i, utf8_buffer, ctx->user_ctx) != READSTAT_HANDLER_OK) {
             retval = READSTAT_ERROR_USER_ABORT;
             goto cleanup;
         }
@@ -625,31 +629,35 @@ static readstat_error_t sav_process_row(unsigned char *buffer, size_t buffer_len
                 col++;
             }
             if (segment_offset == var_info->n_segments) {
-                retval = readstat_convert(ctx->utf8_string, ctx->utf8_string_len, 
-                        ctx->raw_string, raw_str_used, ctx->converter);
-                if (retval != READSTAT_OK)
-                    goto done;
-                value.v.string_value = ctx->utf8_string;
-                if (ctx->value_handler(ctx->current_row, ctx->variables[var_info->index],
-                            value, ctx->user_ctx)) {
-                    retval = READSTAT_ERROR_USER_ABORT;
-                    goto done;
+                if (!ctx->variables[var_info->index]->skip) {
+                    retval = readstat_convert(ctx->utf8_string, ctx->utf8_string_len, 
+                            ctx->raw_string, raw_str_used, ctx->converter);
+                    if (retval != READSTAT_OK)
+                        goto done;
+                    value.v.string_value = ctx->utf8_string;
+                    if (ctx->value_handler(ctx->current_row, ctx->variables[var_info->index],
+                                value, ctx->user_ctx) != READSTAT_HANDLER_OK) {
+                        retval = READSTAT_ERROR_USER_ABORT;
+                        goto done;
+                    }
                 }
                 raw_str_used = 0;
                 segment_offset = 0;
                 var_index += var_info->n_segments;
             }
         } else if (var_info->type == READSTAT_TYPE_DOUBLE) {
-            memcpy(&fp_value, &buffer[data_offset], 8);
-            if (ctx->bswap) {
-                fp_value = byteswap_double(fp_value);
-            }
-            value.v.double_value = fp_value;
-            sav_tag_missing_double(&value, ctx);
-            if (ctx->value_handler(ctx->current_row, ctx->variables[var_info->index],
-                        value, ctx->user_ctx)) {
-                retval = READSTAT_ERROR_USER_ABORT;
-                goto done;
+            if (!ctx->variables[var_info->index]->skip) {
+                memcpy(&fp_value, &buffer[data_offset], 8);
+                if (ctx->bswap) {
+                    fp_value = byteswap_double(fp_value);
+                }
+                value.v.double_value = fp_value;
+                sav_tag_missing_double(&value, ctx);
+                if (ctx->value_handler(ctx->current_row, ctx->variables[var_info->index],
+                            value, ctx->user_ctx) != READSTAT_HANDLER_OK) {
+                    retval = READSTAT_ERROR_USER_ABORT;
+                    goto done;
+                }
             }
             var_index += var_info->n_segments;
             col++;
@@ -1002,7 +1010,10 @@ static readstat_error_t sav_parse_long_value_labels_record(const void *data, siz
         readstat_value_t value = { .type = READSTAT_TYPE_STRING };
         value.v.string_value = value_buffer;
 
-        ctx->value_label_handler(label_name_buf, value, label_buffer, ctx->user_ctx);
+        if (ctx->value_label_handler(label_name_buf, value, label_buffer, ctx->user_ctx) != READSTAT_HANDLER_OK) {
+            retval = READSTAT_ERROR_USER_ABORT;
+            goto cleanup;
+        }
     }
 
 cleanup:
@@ -1229,6 +1240,7 @@ static void sav_set_n_segments_and_var_count(sav_ctx_t *ctx) {
 
 static readstat_error_t sav_handle_variables(readstat_parser_t *parser, sav_ctx_t *ctx) {
     int i;
+    int index_after_skipping = 0;
     readstat_error_t retval = READSTAT_OK;
 
     if (!parser->variable_handler)
@@ -1237,7 +1249,7 @@ static readstat_error_t sav_handle_variables(readstat_parser_t *parser, sav_ctx_
     for (i=0; i<ctx->var_index;) {
         char label_name_buf[256];
         spss_varinfo_t *info = &ctx->varinfo[i];
-        ctx->variables[info->index] = spss_init_variable_for_info(info);
+        ctx->variables[info->index] = spss_init_variable_for_info(info, index_after_skipping);
 
         snprintf(label_name_buf, sizeof(label_name_buf), SAV_LABEL_NAME_PREFIX "%d", info->labels_index);
 
@@ -1245,10 +1257,17 @@ static readstat_error_t sav_handle_variables(readstat_parser_t *parser, sav_ctx_
                 info->labels_index == -1 ? NULL : label_name_buf,
                 ctx->user_ctx);
 
-        if (cb_retval) {
+        if (cb_retval == READSTAT_HANDLER_ABORT) {
             retval = READSTAT_ERROR_USER_ABORT;
             goto cleanup;
         }
+
+        if (cb_retval == READSTAT_HANDLER_SKIP_VARIABLE) {
+            ctx->variables[info->index]->skip = 1;
+        } else {
+            index_after_skipping++;
+        }
+
         i += info->n_segments;
     }
 cleanup:
@@ -1262,7 +1281,7 @@ static readstat_error_t sav_handle_fweight(readstat_parser_t *parser, sav_ctx_t
         for (i=0; i<ctx->var_index;) {
             spss_varinfo_t *info = &ctx->varinfo[i];
             if (info->offset == ctx->fweight_index - 1) {
-                if (parser->fweight_handler(info->index, ctx->user_ctx)) {
+                if (parser->fweight_handler(ctx->variables[info->index], ctx->user_ctx) != READSTAT_HANDLER_OK) {
                     retval = READSTAT_ERROR_USER_ABORT;
                     goto cleanup;
                 }
@@ -1363,7 +1382,7 @@ readstat_error_t readstat_parse_sav(readstat_parser_t *parser, const char *path,
 
     if (parser->info_handler) {
         if (parser->info_handler(ctx->record_count == -1 ? -1 : ctx->row_limit,
-                    ctx->var_count, ctx->user_ctx)) {
+                    ctx->var_count, ctx->user_ctx) != READSTAT_HANDLER_OK) {
             retval = READSTAT_ERROR_USER_ABORT;
             goto cleanup;
         }
@@ -1374,7 +1393,7 @@ readstat_error_t readstat_parse_sav(readstat_parser_t *parser, const char *path,
                         header.file_label, sizeof(header.file_label), ctx->converter)) != READSTAT_OK)
             goto cleanup;
 
-        if (parser->metadata_handler(ctx->file_label, ctx->timestamp, 2, ctx->user_ctx)) {
+        if (parser->metadata_handler(ctx->file_label, ctx->timestamp, 2, ctx->user_ctx) != READSTAT_HANDLER_OK) {
             retval = READSTAT_ERROR_USER_ABORT;
             goto cleanup;
         }
